import logging
import os
import subprocess
import tempfile
from datetime import datetime, timedelta
from pathlib import Path

from django.http import HttpResponse
from django.shortcuts import render

from __settings_local import JAVA_PATH
from geder.settings import BASE_DIR

from .forms import UploadFileForm


# Create your views here.
def airspacecheck_view(request):
    message = None
    file_id = None
    if request.method == "POST":
        form = UploadFileForm(request.POST, request.FILES)
        if form.is_valid():
            file = request.FILES["file"]
            result = run_airspace_check(file)
            message = ""
            if result["result"]:
                short_answer = result["result"].split("=>")[-1].strip()
                if "ok" in short_answer.lower():
                    message += f"Your flight is {short_answer}"
                else:
                    message += (
                        f"Your flight has validated with: {short_answer}"
                        "(check the kml file to be certain)"
                    )
            if result["error"]:
                message += f", the following error occured: {result['error']}"
            file_id = result["file_id"]
        else:
            message = "could not validate file"

    uplaod_form = UploadFileForm()
    context = {"upload_form": uplaod_form, "message": message, "file_id": file_id}
    return render(request, "page_airspacecheck.html", context=context)


def run_airspace_check(file):
    delete_old_directories(os.path.join(BASE_DIR, "flights/"))
    with tempfile.NamedTemporaryFile(
        prefix=os.path.join(BASE_DIR, "flights/"), suffix=".igc", delete=False
    ) as tmpdir:
        file_name = Path(tmpdir.name)
        print(tmpdir.name)
        with open(file_name, "wb+") as destination:
            for chunk in file.chunks():
                destination.write(chunk)

        jar_file = os.path.join(
            BASE_DIR, "stumpfls_airspacecheck", "airspace-check.jar"
        )

        result = subprocess.run(
            [
                f"{BASE_DIR}/stumpfls_airspacecheck/run_web.sh",
                JAVA_PATH,
                jar_file,
                file_name,
            ],
            capture_output=True,  # Python >= 3.7 only
            text=True,  # Python >= 3.7 only
        )
        return {
            "result": result.stdout,
            "error": result.stderr,
            "file_id": str(file_name).split("/")[-1],
        }


def download_file(request, file_id):
    file_path = os.path.join(BASE_DIR, "flights", file_id + ".kml")
    resp = HttpResponse("")

    with open(file_path, "r") as tmp:
        filename = tmp.name.split("/")[-1]
        resp = HttpResponse(tmp, content_type="application/text;charset=UTF-8")
        resp["Content-Disposition"] = "attachment; filename=%s" % filename

    return resp


def delete_old_directories(base_path):
    # Get the current time
    now = datetime.now()

    # Iterate over all items in the base directory
    for entry in os.listdir(base_path):
        entry_path = os.path.join(base_path, entry)

        # Check if the entry is a directory
        if os.path.isfile(entry_path):
            # Get the last modification time of the directory
            last_modification_time = datetime.fromtimestamp(
                os.path.getmtime(entry_path)
            )

            # If the directory is older than one day, delete it
            if now - last_modification_time > timedelta(days=1):
                os.remove(entry_path)
                logging.info(f"Deleted old directory: {entry_path}")
